# GNU Plot script, that displays a graph of the calc_TIR_grip()
# function.
#
# x-axis = miles the tire has travelled
#
# Change the TIRE_? parameters to see how they affect graph.

# tire life in miles
TIRE_max_life  = 50.0

# min grip
TIRE_min_grip  =  0.955

# point at which tire reaches middle of transition
TIRE_sig_midpt =  0.7  * TIRE_max_life

# controls steepness of transition (lower value = steeper transition)
TIRE_sig_trans =  0.15 * TIRE_max_life


scale(x, min_xv, max_xv) = ( ((TIRE_min_grip - 1.0) * x) + (min_xv - (max_xv*TIRE_min_grip)) ) \
                           / (min_xv - max_xv)
sig_func(x)       = 1.0 / (1.0 + exp((x - TIRE_sig_midpt)/TIRE_sig_trans))
sig_min           = sig_func(TIRE_max_life)
sig_max           = sig_func(0.0)
calc_TIRE_grip(x) = scale(sig_func(x), sig_min, sig_max)

sig_start_trans(x) = (x > (TIRE_sig_midpt-TIRE_sig_trans)) ? 1.0 : TIRE_min_grip
sig_mid_trans(x)   = (x > TIRE_sig_midpt) ? 1.0 : TIRE_min_grip
sig_end_trans(x)   = (x > (TIRE_sig_midpt+TIRE_sig_trans)) ? 1.0 : TIRE_min_grip

set yrange [TIRE_min_grip:1.0]
plot [0.0:TIRE_max_life] calc_TIRE_grip(x), sig_start_trans(x), sig_mid_trans(x), sig_end_trans(x)
pause -1